; ZWALLx.ASM : WallRun game for z80asm on ZX80/ZX81-1k
;
;  z80asm zwall#.asm -o zwall#.o	; ZX80 ver
; 
; Take z80fal16.asm (2.11.2015) as base
;  Simple 1 player demo working..
; -> zwall1.asm (3.11.2015)
;  Imp comp player. 
;  Basic working version.
; -> zwall2.asm (3.11.2015)
;  Imp random comp dir. Imp SPEED_DELAY. Prevent bad user dir (into wall).
;  Playable version..
; -> zwall3.asm (4.11.2015)
;  Imp getkey routine, improve key responsiveness.
; -> zwall4.asm (4.11.2015)
;  Imp a title screen. Imp score. Imp complife count.
; -> zwall5.asm (5.11.2015)
;  Imp score, decimal output.
; -> zwall6.asm (6.11.2015)
;  Imp hiscore
; -> zwall7.asm (7.11.2015)
;  Alter text, small changes.
; -> zwall8.asm (8.11.2015)
; -> zwall9.asm (10.11.2015)

USE_ZX80:  equ 0		; Select target comp..
USE_ZX81:  equ 1
USE_EXTRA: equ 1		; Extra features, 2k+ ram
CUSTOM_DISPLAY: equ 0		; custom (flicker-free) display routine - disable to use more-compatible std routines

SPEED_DELAY: equ 8		; Frame count for animation/speed control

PROG_VERSION: equ 0x09

;---------------------------------------------------------
; USE_ZX80 - Header code, Basic stub, etc
;---------------------------------------------------------

if USE_ZX80

org 0x4000


if USE_EXTRA			; Extended version for 2K+..
 PROG_END: equ 0x42af		; Program size
 STACK_TOP: equ 0x47ff		; Stack top 
 VDUX: equ 32
 VDUY: equ 23
else
 PROG_END: equ 0x419f		; Program size
 STACK_TOP: equ 0x43ff		; Stack top 1K
 VDUX: equ 26			; Limit screen size for 1k
 VDUY: equ 22
endif


NO_TARGETS: equ 2		; No of targets

pVdu:    equ 400Ch		; Ptr to hires screen (D_FILE)
pEnd:    equ 4010h		; Ptr to screen end (DF_END)
zframes: equ 401eh		; Frame counter (lo first, 1/50 sec)	
zseed:   equ 401ch		; Random seed

io_vsync: equ 0xfe		; Vsync switch

CH_NEWLINE: equ 0x76		; New line char
CH_SPACE:   equ 0
CH_PLAYER:  equ 9
CH_COMP:    equ 128
CH_TARGET:  equ 13
CH_EDGE:    equ 61+128
CH_ZERO:    equ 28

zx_getch:  equ 0x013f		; Display screen, read a char.
zx_disp2:  equ 0x01ad		; Display part of screen

playpos:   equ 0x4028		; Player pos (ptr)
playdir:   equ 0x402a		; Dir (16 bit signed)
comppos:   equ 0x402c		; Comp pos
compdir:   equ 0x402e
level:     equ 0x4030		; Game Level
randseed:  equ 0x4032		; 3 byte seed for io_rand routine 
complife:  equ 0x4035
score:     equ 0x4036

datastart:			; $4000-$4028 : System variables
 db 0xff,0x88,0xfe,0xff,0xfc,0x40,0xe7,0x03
	; System word ptrs at 0x4008..
 ;   VARS     E_LINE	   D_LINE      DF_EA         DF_END
 dw PROG_END, PROG_END+1, PROG_END+3, PROG_END+4, PROG_END+4

			; Assort system vars
 db 0x02,0x00,0x00,0x00,0x00,0x00
 db 0x00,0x00,0xb0,0x07,0x00,0x00,0x7c,0x3b
 db 0x00,0x00,0x00,0x00,0x21,0x17,0xfc,0x40

			; Basic stub:10 RAND USR(16443)
 db 0x00,0x0a,0xef,0x3a,0x38,0x37,0xda,0x1d,0x22,0x20,0x20,0x1f,0xd9,0x76
			; Basic:  999 REM <RET><RET><CODESTART>..   (Double <RET> prevent list of m/code)
 db 0x03,0xe7,0xfe,0x76,0x76


endif		; USE_ZX80

;---------------------------------------------------------
; USE_ZX81 - Header code, Basic stub, etc
;---------------------------------------------------------

if USE_ZX81

org 0x4009



if USE_EXTRA			; Extended version for 2K+..
 PROG_END:  equ 0x42ff		; Program size
 STACK_TOP: equ 0x47ff		; Stack top 2K+
 VDUX: equ 32
 VDUY: equ 23
else
 if CUSTOM_DISPLAY
  PROG_END:  equ 0x421f		; Program size
  VDUX: equ 22
 else
  PROG_END:  equ 0x41ef		; Program size
  VDUX: equ 23
 endif
 STACK_TOP: equ 0x43ff		; Stack top 1K
 VDUY: equ 22
endif

NO_TARGETS: equ 2		; No of targets

pVdu:    equ 400Ch		; Ptr to hires screen (D_FILE)
pVars:   equ 4010h		; Ptr to basic vars (VARS)
zframes: equ 4034h		; Frame counter (lo first, 1/50 sec)	

io_vsync: equ 0xfd		; Vsync switch

CH_NEWLINE: equ 0x76		; New line char
CH_SPACE:   equ 0
CH_PLAYER:  equ 8
CH_COMP:    equ 128
CH_TARGET:  equ 13
CH_EDGE:    equ 61+128
CH_ZERO:    equ 28

zx_getch:  equ 0x013f		; Display screen, read a char.
zx_disp2:  equ 0x02b5		; Display part of screen

randseed:  equ 0x403c		; 3 byte seed for io_rand routine 
playpos:   equ 0x4040		; Player pos (ptr)
playdir:   equ 0x4042		; Dir (16 bit signed)
comppos:   equ 0x4044		; Comp pos
compdir:   equ 0x4046
level:     equ 0x4048		; Game Level
complife:  equ 0x4049
score:     equ 0x404a


datastart:			; $4009 - $407D  System variables
  db 0x00,0x0a,0x00
  dw (PROG_END-0x001a)		; $400C  D_File  Ptr Video/end of Basic
  dw (PROG_END-0x0019)		; $400E  DF_CC   Ptr VRAM loc for PRINT
  dw (PROG_END-0x0001)		; $4010  VARS    Ptr BASIC Variables
  db 0,0
  dw (PROG_END)			; $4014  E_LINE  Ptr End of Save Area
  dw (PROG_END+0x0004)
  db 0,0
  dw (PROG_END+0x0005)		; $401A  STKBOT  Ptr BASIC Calculator Stack
  dw (PROG_END+0x0005)		; $401C  STKEND  Ptr bottom of Machine/Calc Stack

  ;db 0xc5,0x41,0xc6,0x41,0xde,0x41,0x00,0x00,0xdf,0x41,0xe3,0x41
  ;db 0x00,0x00,0xe4,0x41,0xe4,0x41
  db 0x00,0x5d,0x40
  db 0x00,0x02,0x00,0x00,0xbf,0xfd,0xff,0x37
  dw (PROG_END-0x001a)
  ;db 0xc5,0x41
  db 0x00,0x00,0x00,0x00,0x00,0x8d
  db 0x0c,0x00,0x00,0x22,0xad,0x00,0x00,0xbc
  db 0x21,0x18,0x40,0x00,0x00,0x00,0x00,0x00
  db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
  db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
  db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
  db 0x00,0x00,0x00,0x76,0x00,0x00,0x00,0x00
  db 0x00,0x00,0x84,0x00,0x00,0x00,0x84,0xa0
  db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
  db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
  db 0x00,0x00,0x00,0x00

  db 0x00,0x0a			; Start of basic:  10 REM..  
  ;db 0x32,0x01
  dw (PROG_END-0x40ad)		; Variable rem statement len
  db 0xea


endif			; USE_ZX81

;---------------------------------------------------------
; codestart - main code
;---------------------------------------------------------

codestart:			; Called by USR(16443)
  ld sp,STACK_TOP		; Set SP to top of RAM

 if USE_ZX81
  out (0fdh),a			; Set fast mode 
 endif
 if USE_EXTRA
   jp init_code		; Title screen, other init..
 endif
  
new_game:			; Start a new game
  xor a
  ld (level),a		; Init level count
  ld h,a
  ld l,a
  ld (score),hl		; Clr score

new_level:		; Start a new level
  ld hl,player_start
  ld (playpos),hl 	; Init player pos
  ld hl,1
  ld (playdir),hl 	; Init player dir
  dec hl
  dec hl
  ld (compdir),hl 	; Init comp dir (= -1)
  ld hl,comp_start
  ld (comppos),hl 	; Init comp pos 

  ld hl,level
  inc (hl)		; level ++; 

  ld hl,dfile_new	; Reuse basic stub code space
  ld (pVdu),hl		; Update D_FILE pointer

  ld a, CH_NEWLINE	; Place CH_NEWLINE at start of screen.
  ld (hl),a
  inc hl
  ld (hl),a
  inc hl
  
  ld c,VDUY		; Y=24 (C)

loopy:			; Outer loop:  for(y=VDUY;y;y--)..
  ld b,VDUX
  ld a,c
  dec a
  dec a
  cp VDUY-2		; Put edge char on top/bot line
  ld a,CH_SPACE
  jr c,loopx		; Carry set, not border
  ld a,(level)		; Display mine fall rate (score) in edge
  add a,CH_ZERO+128
  and 0xbf
  ; ld a,CH_EDGE

loopx:			; Inner loop - fill screen line:  for(x=VDUX;x;x--)..
  ld (hl),a		; *pVdu = a
  inc hl		; pVdu ++;
  djnz loopx		; X--; if (X>0) goto loopx; (B)

  ld a, CH_NEWLINE	; Add NEWLINE at the end of each line
  ld (hl),a
  inc hl

  dec c				
  jr nz,loopy		; Y--; if (Y>0) goto loopy; (C)

  ld (hl),a		; Add NEWLINE
  inc hl
  ld (hl),a		; Add NEWLINE
  ;ld (pVars),hl

 if USE_ZX81
  out (0feh),a		; Set slow mode
 endif

  ld a,(level)		; Draw Targets/Obstacles
  ld d,a
  inc a
  srl a
  ld (complife),a	; Comp life count = (level/2)+1
targloop:
  call io_rand
  ld (hl),CH_TARGET
  ;call io_rand
  ;ld (hl),CH_EDGE
  dec d
  jr nz,targloop

  if USE_EXTRA
   call score_add	; Display score, score += level..
  endif

playloop:		; Main game play loop
  ;ld hl,(playpos)	; Draw player char
  ;ld (hl),CH_PLAYER

if CUSTOM_DISPLAY

keyloop:

  ld de,0
  ld b,SPEED_DELAY		; Delay for a few frames
display_loop:
  push bc
  push de
  call displayframe_delay	; Flicker-free display
  ;in a,(0xfe)			; Read keyboard (needed for vert sync?)
  call getkey
  jr z,d1
  pop bc
  push de
d1:
  pop de
  pop bc
  djnz display_loop		

  ld a,d
  or e
  jr z,keepdir		; No key down..

else			; (CUSTOM_DISPLAY == 0) - Use std routines..

keyloop:

if USE_ZX80

waitrelease:		; Scan keyboard, wait till key release
  ld bc,0x00fe
  in a,(c)
  cpl
  and 0x1f
  jr nz,waitrelease

  call zx_getch		; Read keyboard, display..

endif

if USE_ZX81

  ld hl,zframes		; Delay for a few frames
  ld a,(hl)
  sub SPEED_DELAY
display_loop:
  cp (hl)
  jr nz,display_loop

endif

  call getkey		; Get dir in (DE)
  jr z,keepdir		; No key down..

endif			; (CUSTOM_DISPLAY)


moveplayer:			; Update player pos 
  ld hl,(playpos)
  add hl,de			; playpos += movedir
  ld a,(hl)			; Have we hit anything?
  or a
  jr z,drawplayer		; Empty sqr..
  cp CH_TARGET
  jr z,play_score
  ld a,(playdir)		; is dir in (DE) same as one in (playdir)
  cp e
  jp z,new_game			; No new dir, end of game
				; New dir hits obstacle, so try orig dir
keepdir:
  ld de,(playdir)		; Keep cur dir (discard any new dir)
  jr moveplayer	

play_score:			; Target hit..
 if USE_EXTRA
  push hl
  push de
  call score_add
  pop de
  pop hl
 endif

drawplayer:
  ld (playdir),de		; Save (new?) dir
  ld (playpos),hl		; Save new player position
  ld (hl),CH_PLAYER		; Draw player 

compmove:
  ld hl,(comppos)
  ld de,(compdir)
  add hl,de			; playpos += movedir
  ld a,(hl)			; Have we hit anything?
  or a
  jr z,drawcomp

  ld b,4			; Scan a table of 4 dirs to find clear comp dir..
  ld hl,tabledir
  ld a,(randseed)		; Randomise direction by shifting starting point in lookup
  and 1
  jr nz,finddir	
  inc hl
  inc hl			; Start at (tabledir+2)
finddir:
  ld e,(hl)			; Load dir at (HL) into (DE)
  inc hl
  ld d,(hl)
  inc hl
  ld (compdir),de		; Save it.
  push hl
  ld hl,(comppos)
  add hl,de
  ld a,(hl)			; Load dest for this dir..
  pop hl
  or a
  jr z,compmove			; Found a clear dir, move comp
  djnz finddir		
				; All 4 dirs blocked, comp loses a life
  ld hl,complife
  dec (hl)
  jp z,new_level		; All lifes gone, next level

  call io_rand			; Gen new rnd comp position

drawcomp:
  ld (comppos),hl
  ld (hl),CH_COMP		; Draw comp move
  
  jp playloop


tabledir:			; Lookup table of 4 possible dirs (+1 to help rand fn)
  dw VDUX+1, 0xffff-VDUX, 0x0001, 0xffff, VDUX+1


if USE_EXTRA

hiscore: db 0,0

score_add:
  ld a,(level)
  ld hl,(score)
  ld c,a
  ld b,0
  add hl,bc
  ld (score),hl
  push hl
  ld bc,(hiscore)
  sbc hl,bc
  pop hl
  jr c,score1
  ld (hiscore),hl
score1:
  ld de,screen_hiscore
  ld hl,(hiscore)
  call output_int
  ld hl,0x0e0e
  ;ld hl,0x2e2d
  ld (screen_hiscoret),hl
  ld (screen_hiscoret2),hl
  
score_show:		; Ouput score 
  ld de,screen_score
  ld hl,(score)

output_int:		; Output decimal integer in (HL) to loc (DE)	
  ld bc,1000
  call output_digit
  ld bc,100
  call output_digit
  ld c,10
  call output_digit
  ld c,1		; Drop thro last digit..

output_digit:		; Output a digit
  scf
  ccf
  ld a,CH_ZERO-1
output_d1:
  inc a
  sbc hl,bc
  jr nc,output_d1	; Count decimal digits..
  add hl,bc
  and 0x3f		; Digit in (A)
  ld (de),a
  inc de
  ret

endif

;---------------------------------------------------------
; getkey - get compass dir in (DE), set zero flag if none
;---------------------------------------------------------

getkey:		; Directly read + decode ZX81/ZX80 Keyboard Matrix, ret dir in (DE), or zero-flag if none  

	;  Port____Line____Bit__0____1____2____3____4__
	;  FEFEh  0  (A8)     SHIFT  Z    X    C    V
	;  FDFEh  1  (A9)       A    S    D    F    G
	;  FBFEh  2  (A10)      Q    W    E    R    T
	;  F7FEh  3  (A11)      1    2    3    4    5
	;  EFFEh  4  (A12)      0    9    8    7    6
	;  DFFEh  5  (A13)      P    O    I    U    Y
	;  BFFEh  6  (A14)    ENTER  L    K    J    H
	;  7FFEh  7  (A15)     SPC   .    M    N    B

			; Use keys about 'S' (QWEASDZXC) for compass dir movement
  ld de,0xfffd-VDUX	; de= movedir offset (-VDUX-3)
  ld bc,0xfbfe		; Row "qwert"
  in a,(c)		; Read keyboard
  cpl
  and 2			; Flip & Mask bits 0,1,2
  jr nz,key3		; Key hit in (A).. (q=1,w=2,e=4..)

  ld de,0xfffe		; movedir offset  (-2)
  ld b,0xfd		; Row "asdfg"
  in a,(c)		; Read keyboard
  cpl
  and 5	
  jr nz,key3

  ld de,VDUX-1		; movedir offset
  ld b,0xfe		; Row "Szxcv"
  in a,(c)		; Read keyboard
  srl a			; Bring "zxc" to lo bits
  cpl
  and 2
  ret z
  ;jr z,keepdir		; No key, keep cur dir

key3:			; code 1,2,4 in (A)
  cp 4
  jr nz,key4
  dec a
key4:
  add a,e		; Add X-Offset (1,2,3) in (A) to dir in (DE)
  ld e,a
  jr nc,key5
  inc d
key5:			; New dir in (DE)
  or 1			; Clr zero flag
  ret

;---------------------------------------------------------
; io_rand - gen rand screen pos
;---------------------------------------------------------

		; A simple psuedo random fn. Return empty screen pos in (HL)
io_rand:
  ld e,127		; Count tries
io_rand1:
  ld a,(randseed)
  ld bc,(randseed+1)
  dec bc
  ld (randseed),bc
  rla 
  rla 
  adc a,b
	; ld h,start/256 :ld l,a : adc a,(hl)   ; a = a ^ PEEK (start + a)
  ld (randseed+2),a
	; Random numbers in A, BC. Now convert to screen pos in (HL)
  ld hl,screen_start

if USE_EXTRA			; Extend rand to cover larger screen
  srl a				; A=A/2, C=rand 0,1
  jr c, io_rand2
  ld hl,screen_start2		; Alt position to cover bottom of screen
io_rand2:
endif

  and 1
  ld b,a
  add hl,bc
  ld a,(hl)			; Rand pos in (HL) 
  dec e
  ret z				; Limit to 256 tries.. (e=0 means fail)
  or a
  jr nz, io_rand1		; If occupied, try again..
  ret

;---------------------------------------------------------
; USE_ZX80 - Terminating code
;---------------------------------------------------------

if USE_ZX80

if CUSTOM_DISPLAY

displayframe_delay:	; Custom flicker-free display routine
  ld b,64
delay1:
  djnz delay1		; Delay loop during vert retrace

displayframe:
  out (io_vsync),a		; Vert sync off
  
  ld hl,dfile_hi	; Display file loc (bit 15 set)
  ld a,$e9		; line timing count (was $ed) (alter to shift horiz screen pos)
  ld b,25		; 24 rows + 1 for top
  call zx_disp2 	; ($01ad): ROM routine to display top/main screen

  ld a,$f0		; line timing count
  dec hl		; HL points to HALT
  inc b			; Set row count to 1
  dec (iy+$23)		; 47 lines in bottom border
  call zx_disp2		; ($01ad): Show bottom part

  ld (iy+$23),30h	; Reset count
  in a,(io_vsync)	; Vert sync on (Comment out if reading keyboard)
  ret


endif			; (CUSTOM_DISPLAY)

dfile_new:	; New loc for dfile - reuse unused Basic stub code space.

screen_score: equ dfile_new +2
screen_hiscoret: equ dfile_new + 6
screen_hiscore: equ dfile_new + 8
screen_hiscoret2: equ dfile_new + 12
screen_start: equ dfile_new + VDUX+2
screen_start2: equ dfile_new + (VDUX * VDUY) - 512

dfile_hi: equ dfile_new + 0x8000		; Display file himem loc (bit 15 set)

		; Player start center pos Constant (dfile_new must be first)
player_start: equ dfile_new+((VDUX+1)*(VDUY/2))+(VDUX/4)
comp_start:   equ dfile_new+((VDUX+1)*(VDUY/2))+VDUX-(VDUX/4)+2

if USE_EXTRA		; Title screen, one-time initialisation..

init_code:
  ld hl,title_screen
  ld (pVdu),hl

title_loop:		; Scan keyboard, wait till key release

 if USE_ZX80
  call zx_getch		; Read keyboard, display..
 endif

  ld bc,0x40fe
  in a,(c)
  cpl
  and 0x1f
  jr z,title_loop

  jp new_game		; Run game

title_screen:
  db 0x76, 0x3c,0x26,0x31,0x31,0x37,0x3a, 0x33,0x00,0x1d,0x1b,0x1c,0x1c+PROG_VERSION,0x00,	 0x76,0x76	; WALLRUN 1.0x
  db 0x10,0x28,0x11,0x26,0x1b,0x32,0x2e,0x31,0x31, 0x2a,0x39,0x39,0x00,0x1e,0x1c,0x1d,0x21
  db 0x1b,0x00,0x2b,0x37,0x2a,0x2a,0x3c,0x26, 0x37,0x2a,0x1b,  0x76,0x76			; .. FREEWARE.
  db 0x3a,0x38,0x2a,0x00,0x30,0x2a,0x3e,0x38,0x00,0x01,0x3c,0x26, 0x29,0x3d,0x01		; USE KEYS..
  db 0, 0x39,0x34,0, 0x28,0x34,0x33,0x39,0x37,0x34,0x31, 0x76,
  db 0x3e,0x34,0x3a,0x37, 0x00,0x35,0x31,0x26,0x3e,0x2a,0x37,0,0x26,0x33,0x29,0
  db 0x34,0x3a,0x39,0x16,0x3c,0x26,0x31,0x31, 0x00,0x39,0x2d,0x2a, 0x76
  db 0x28,0x34,0x32,0x35,0x3a,0x39,0x2a,0x37,0x1b,0x00 ,0x76,0x76
  db 0x2d,0x2e,0x39,0x00,0x38,0x35,0x26,0x28,0x2a				; HIT SPACE
  db 0x76,0x76,0x76,0x76,0x76,0x76,0x76,0x76
  db 0x76,0x76,0x76,0x76,0x76,0x76,0x76,0x76
  db 0x76,0x76
  
endif

		; Remains of basic stub code. Note:- line length must match for code to run.
 
 ds (PROG_END-1-$)	; Pad unused REM space upto PROG_END with zeros
 db 0x76		; Term REM--..  line

 db 0x80		; Var table terminator

endif		; USE_ZX80

;---------------------------------------------------------
; USE_ZX81 - Terminating code
;---------------------------------------------------------

if USE_ZX81

if CUSTOM_DISPLAY

displayframe_delay:	; Custom flicker-free display routine
  ld b,255
delay1:
  djnz delay1		; Delay loop during vert retrace

displayframe:
  out (io_vsync),a		; Vert sync off
  
  ld hl,dfile_hi	; Display file loc (bit 15 set)
  ld a,$e6		; line timing count (was $ed) (alter to shift horiz screen pos)
  ld c,48		; 48 lines in the bottom border.
  ld b,25		; 24 rows + 1 for top
  call zx_disp2 	; ($02b5): ROM routine to display top/main screen

  ld a,$e7		; line timing count
  dec hl		; HL points to HALT
  inc b			; Set row count to 1
  ld c,47		; 47 lines in the bottom border.
  call zx_disp2		; ($02b5): Show bottom part

  ;ld (iy+$23),30h	; Reset count
  in a,(io_vsync)	; Vert sync on (Comment out if reading keyboard)
  ret

endif			; (CUSTOM_DISPLAY)


dfile_new:	; New loc for dfile - reuse unused Basic stub code space.

screen_score: equ dfile_new +2
screen_hiscoret: equ dfile_new + 6
screen_hiscore: equ dfile_new + 8
screen_hiscoret2: equ dfile_new + 12
screen_start: equ dfile_new + VDUX+2
screen_start2: equ dfile_new + (VDUX * VDUY) - 512

dfile_hi: equ dfile_new + 0x8000		; Display file himem loc (bit 15 set)
		; Player start center pos Constant (dfile_new must be first)
player_start: equ dfile_new+((VDUX+1)*(VDUY/2))+VDUX/4
comp_start:   equ dfile_new+((VDUX+1)*(VDUY/2))+2+VDUX-(VDUX/4)

if USE_EXTRA		; Title screen, one-time initialisation..

init_code:
  ld hl,title_screen
  ld (pVdu),hl
 if USE_ZX81
  out (0feh),a		; Set slow mode
 endif

title_loop:		; Scan keyboard, wait till key release

 if USE_ZX80
  call zx_getch		; Read keyboard, display..
 endif

  ld bc,0x40fe
  in a,(c)
  cpl
  and 0x1f
  jr z,title_loop

 if USE_ZX81
  out (0fdh),a		; Set fast mode 
 endif
  jp new_game		; Run game

	;ZX81  - Chrset:
	;  ____0___1___2___3___4___5___6___7___8___9___A___B___C___D___E___F____
	;  00 SPC [' ][ '][''][. ][: ][.'][:']{::}{..}{''} "  GBP  $   :   ?  0F
	;  10  (   )   >   <   =   +   -   *   /   ;   ,   .   0   1   2   3  1F
	;  20  4   5   6   7   8   9   A   B   C   D   E   F   G   H   I   J  2F
	;  30  K   L   M   N   O   P   Q   R   S   T   U   V   W   X   Y   Z  3F

title_screen:
  db 0x76,0, 0x3c,0x26,0x31,0x31,0x37,0x3a, 0x33,0x00,0x1d,0x1b,0x1c,0x1c+PROG_VERSION,0x00,	 0x76,0x76	; WALLRUN 1.0x
  db 0x10,0x28,0x11,0x26,0x1b,0x32,0x2e,0x31,0x31, 0x2a,0x39,0x39,0x00,0x1e,0x1c,0x1d,0x21
  db 0x1b,0x00,0x2b,0x37,0x2a,0x2a,0x3c,0x26, 0x37,0x2a,0x1b,  0x76,0x76			; .. FREEWARE.
  db 0x3a,0x38,0x2a,0x00,0x30,0x2a,0x3e,0x38,0x00,0x0b,0x3c,0x26, 0x29,0x3d,0x0b		; USE KEYS..
  db 0, 0x39,0x34,0, 0x28,0x34,0x33,0x39,0x37,0x34,0x31, 0x76,
  db 0x3e,0x34,0x3a,0x37, 0x00,0x35,0x31,0x26,0x3e,0x2a,0x37,0,0x26,0x33,0x29,0
  db 0x34,0x3a,0x39,0x16,0x3c,0x26,0x31,0x31, 0x00,0x39,0x2d,0x2a, 0x76
  db 0x28,0x34,0x32,0x35,0x3a,0x39,0x2a,0x37,0x1b,0x00, 0x76,0x76
  db 0x2d,0x2e,0x39,0x00,0x38,0x35,0x26,0x28,0x2a				; HIT SPACE
  db  0x76,0x76,0x76,0x76,0x76,0x76,0x76,0x76
  db  0x76,0x76,0x76,0x76,0x76,0x76,0x76,0x76
  db  0x76,0x76
  
endif


		; Remains of basic stub code (10 REM ----..) Note:- line length must match for code to run.
  ds PROG_END-45-$	; Pad unused REM space with zeros
  db  0x76		; Term 10 REM--..  line
			; Basic code: 20 RAND USR 16514
  db  0x00,0x14,0x0e,0x00,0xf9,0xd4,0x1d,0x22
  db  0x21,0x1d,0x20,0x7e,0x8f,0x01,0x04,0x00,0x00,0x76
			; Display file data.. (25 newlines)
  db  0x76,0x76,0x76,0x76,0x76,0x76,0x76,0x76
  db  0x76,0x76,0x76,0x76,0x76,0x76,0x76,0x76
  db  0x76,0x76,0x76,0x76,0x76,0x76,0x76,0x76
  db  0x76
			; Var table terminator
  db  0x80


endif		; USE_ZX81

end

